package de.gc.db.jdbi.oop.bespos;

import java.util.List;
import java.util.stream.Stream;

import org.jdbi.v3.core.mapper.JoinRow;
import org.jdbi.v3.sqlobject.config.RegisterBeanMapper;
import org.jdbi.v3.sqlobject.config.RegisterConstructorMapper;
import org.jdbi.v3.sqlobject.config.RegisterJoinRowMapper;
import org.jdbi.v3.sqlobject.customizer.Bind;
import org.jdbi.v3.sqlobject.customizer.BindBean;
import org.jdbi.v3.sqlobject.statement.SqlQuery;
import org.jdbi.v3.sqlobject.statement.SqlUpdate;

import de.gc.db.jdbi.oop.artikel.Artikel;
import de.gc.db.jdbi.oop.kunde.Kunde;

public interface BesPosDAO {

   @SqlUpdate("""
         CREATE TABLE IF NOT EXISTS bespos (
            aid     INT,
            kid     INT
         ) ENGINE=InnoDB DEFAULT CHARSET=utf8
         """)
   void createTable();

   @SqlQuery("SELECT * FROM bespos")
   @RegisterBeanMapper(BesPos.class)
   List<BesPos> getAll();

   @SqlQuery("""
         SELECT k.id AS k_id, k.name AS k_name, k.oid AS o_id,
                o.name AS o_name, o.plz AS o_plz,
                a.id AS a_id, a.name AS a_name
            FROM bespos bp INNER JOIN kunde k   ON bp.kid=k.id
                           INNER JOIN artikel a ON bp.aid=a.id
                           LEFT JOIN  ort o     ON k.oid=o.id
            ORDER BY k.id
         """)
   @RegisterConstructorMapper(Kunde.class)
   @RegisterConstructorMapper(Artikel.class)
   @RegisterJoinRowMapper({ Kunde.class, Artikel.class })
   Stream<JoinRow> getKundeArtikel();

   @SqlUpdate("""
         INSERT INTO bespos (aid, kid)
            VALUES (:aid, :kid)
         """)
   void insert(@BindBean BesPos bp);

   @SqlUpdate("""
         INSERT INTO bespos (aid, kid)
            VALUES (:aid, :kid)
         """)
   void insert(@Bind("aid") int aid, @Bind("kid") int kid);
}
